;;LISP99

;;p08
(defun compress (ls)
  (compress1 (car ls) (cdr ls)))

(defun compress1 (x ls)
  (cond ((null ls) nil)
        ((eq x (car ls)) (compress1 x (cdr ls)))
        (t (cons x (compress1 (car ls) (cdr ls))))))

;;p21
(defun insert-at (x ls n)
  (if (= n 1)
      (cons x ls)
      (cons (car ls) (insert-at x (cdr ls) (- n 1)))))

;;p34
(defun totient-phi (n)
  (length (divisors n)))

(defun divisors (n)
  (divisors1 nil 1 n))

(defun divisors1 (ls m n)
  (cond ((= m n) (cons m ls))
        ((= (mod n m) 0) (divisors1 (cons m ls) (+ m 1) n))
        (t (divisors1 ls (+ m 1) n))))


;;p01
(defun my-last (x)
  (if (null (cdr x))
      (car x)
      (my-last (cdr x))))

;(defun my-last (x)
;  (car (reverse x)))

;(defun my-last (x)
;  (while (not (null (cdr x)))
;         (setq x (cdr x)))
;  (car x))

;;p03
(defun element-at (x n)
  (elt x (- n 1)))

;;p07
(defun my-flatten (x)
  (cond ((null x) nil)
        ((atom x) (list x))
        (t (append (my-flatten (car x)) (my-flatten (cdr x))))))


;;p09
(defun pack (x)
  (let ((y (pack1 (car x) '() x)))
    (if (null (cdr y))
        y
        (cons (car y) (pack (cdr y))))))

(defun pack1 (x y z)
  (cond ((null z) (cons y nil))
        ((not (eq x (car z))) (cons y z))
        (t (pack1 x (cons (car z) y) (cdr z)))))

;;p10
(defun encode (x)
  (mapcar (lambda (y) (list (length y)(car y)))
          (pack x)))

;;p11
(defun encode-modified (x)
  (mapcar (lambda (y) (if (= (length y) 1)
                          (car y)
                          (list (length y)(car y))))
          (pack x)))

;;p12
(defun decode (x)
  (mapcan (lambda (y) (if (atom y)
                          (list y)
                          (decode1 (elt y 0) (elt y 1))))
          x))

(defun decode1 (n x)
  (if (= n 0)
      '()
      (cons x (decode1 (- n 1) x))))

;;p13
(defun encode-direct (x)
  (let ((y (encode-direct1 (car x) 0 x)))
    (if (null (cdr y))
        (if (= (elt (car y) 0) 0)
            (car (car y))
            y)
        (cons (car y) (encode-direct (cdr y))))))

(defun encode-direct1 (x n z)
  (cond ((null z) (cons (list n x) nil))
        ((not (eq x (car z))) (cons (list n x) z))
        (t (encode-direct1 x (+ n 1) (cdr z)))))

(defun ackermann (x y)
  (if (= x 0)
      (+ y 1)
      (if (= y 0  )
          (ackermann (- x 1) 1)     
          (ackermann (- x 1)
                     (ackermann x (- y 1)) ))))

(defun union (x y)
  (cond ((null x) y)
        ((member (car x) y) (union (cdr x) y))
        (t (union (cdr x) (cons (car x) y)))))

(defun intersection (x y)
  (cond ((null x) nil)
        ((member (car x) y) (cons (car x) (intersection (cdr x) y)))
        (t (intersection (cdr x) y))))

;;p17
;;recursive
(defun split (x n)
  (split1 nil x n))

(defun split1 (x y n)
  (cond ((null y) nil)
        ((= n 0) (list (reverse x) y))
        (t (split1 (cons (car y) x) (cdr y) (- n 1)))))

;;imperative
;(defun split (x n)
;  (for ((front nil (cons (car rear) front))
;       (rear x (cdr rear))
;      (m n (- m 1)))
;     ((= m 0) (list (reverse front) rear))))

;;p16
(defun drop (x n)
  (drop1 x nil n 1))

(defun drop1 (x y n m)
  (cond ((null x) (reverse y))
        ((= n m)(drop1 (cdr x) y n 1))
        (t (drop1 (cdr x) (cons (car x) y) n (+ m 1)))))

;;p31
(defun is-prime (n)
  (labels ((iter (m limit)
                 (cond ((> m limit) t)
                       ((= (mod n m) 0) nil)
                       (t (iter (+ m 2) limit)))))
    (cond ((< n 2) nil)
          ((= (mod n 2) 0) nil)
          (t (iter 3 (isqrt n))))))

;;p02
(defun my-but-last (x)
  (if (< (length x) 2)
      (error "my-but-last: illegal argument" x)
      (my-but-last1 x)))

(defun my-but-last1 (x)
  (if (null (cdr (cdr x)))
      x
      (my-but-last1 (cdr x))))

;;p22
;recursive
(defun range (m n)
  (if (>= m n)
      nil
      (cons m (range (+ m 1) n))))
;imperative
;(defun range (m n)
;  (for ((m1 m (+ m1 1))
;        (ls nil (cons m1 ls)))
;       ((>= m1 n) (reverse ls))))

;;p35
(defun prime-factors (n)
  (labels ((iter (p x ls z)
                 (cond ((> p z) (cons x ls))
                       ((= (mod x p) 0) 
                        (let ((n1 (div x p)))
                          (iter 2 n1 (cons p ls) (isqrt n1))))
                       ((= p 2) (iter 3 x ls z))
                       (t (iter (+ p 2) x ls z)))))
    (cond ((< n 0) nil)
          ((< n 2) (list n))
          (t (iter 2 n '() (isqrt n))))))

;;p36
(defun prime-factors-mult (n)
  (prime-factors-mult1 (prime-factors n)))

(defun prime-factors-mult1 (x)
  (let ((y (prime-factors-mult2 (car x) 0 x)))
    (if (null (cdr y))
        y
        (cons (car y) (prime-factors-mult1 (cdr y))))))

(defun prime-factors-mult2 (x n z)
  (cond ((null z) (cons (list x n) nil))
        ((not (= x (car z))) (cons (list x n) z))
        (t (prime-factors-mult2 x (+ n 1) (cdr z)))))

;;p32
(defun my-gcd (x y)
  (cond ((< x y) (my-gcd y x))
        ((= (mod x y) 0) y)
        (t (my-gcd y (mod x y)))))

;;p33
(defun coprime (x y)
  (= (my-gcd x y) 1))


